@@ -5,15 +5,17 @@
 
 Entity is the base of all RAMSES-II objects: devices and also system/zone constructs.
 """
+from __future__ import annotations
 
 import asyncio
 import logging
 import random
+from asyncio import Future
 from datetime import datetime as dt
 from datetime import timedelta as td
 from inspect import getmembers, isclass
 from sys import modules
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any
 
 from .const import (
     SZ_ACTUATORS,
@@ -26,9 +28,10 @@ from .const import (
     __dev_mode__,
 )
 from .protocol import CorruptStateError, Message
+from .protocol.frame import _CodeT, _DeviceIdT, _HeaderT, _VerbT
 from .protocol.ramses import CODES_SCHEMA
 from .protocol.transport import PacketProtocolPort
-from .schema import SZ_CIRCUITS
+from .schemas import SZ_CIRCUITS
 
 # skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
@@ -40,95 +43,16 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     FA,
     FC,
     FF,
+    Code,
 )
 
-# skipcq: PY-W2000
-from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
-    _0001,
-    _0002,
-    _0004,
-    _0005,
-    _0006,
-    _0008,
-    _0009,
-    _000A,
-    _000C,
-    _000E,
-    _0016,
-    _0100,
-    _0150,
-    _01D0,
-    _01E9,
-    _0404,
-    _0418,
-    _042F,
-    _0B04,
-    _1030,
-    _1060,
-    _1081,
-    _1090,
-    _1098,
-    _10A0,
-    _10B0,
-    _10E0,
-    _10E1,
-    _1100,
-    _11F0,
-    _1260,
-    _1280,
-    _1290,
-    _1298,
-    _12A0,
-    _12B0,
-    _12C0,
-    _12C8,
-    _12F0,
-    _1300,
-    _1F09,
-    _1F41,
-    _1FC9,
-    _1FCA,
-    _1FD0,
-    _1FD4,
-    _2249,
-    _22C9,
-    _22D0,
-    _22D9,
-    _22F1,
-    _22F3,
-    _2309,
-    _2349,
-    _2389,
-    _2400,
-    _2401,
-    _2410,
-    _2420,
-    _2D49,
-    _2E04,
-    _2E10,
-    _30C9,
-    _3110,
-    _3120,
-    _313F,
-    _3150,
-    _31D9,
-    _31DA,
-    _31E0,
-    _3200,
-    _3210,
-    _3220,
-    _3221,
-    _3223,
-    _3B00,
-    _3EF0,
-    _3EF1,
-    _PUZZ,
-)
 
 _QOS_TX_LIMIT = 12  # TODO: needs work
 
 DEV_MODE = __dev_mode__ and False
 
+# USE_JMESPATH = False
+
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
@@ -147,7 +71,7 @@ def class_by_attr(name: str, attr: str) -> dict:  # TODO: change to __module__
         getattr(c[1], attr): c[1]
         for c in getmembers(
             modules[name],
-            lambda m: isclass(m) and m.__module__ == name and getattr(m, attr, None),
+            lambda m: isclass(m) and m.__module__ == name and getattr(m, attr, None),  # type: ignore[arg-type, return-value]
         )
     }
 
@@ -155,9 +79,13 @@ def class_by_attr(name: str, attr: str) -> dict:  # TODO: change to __module__
 class MessageDB:
     """Maintain/utilize an entity's state database."""
 
+    _gwy: Any  # HACK
+    ctl: Any  # HACK
+    tcs: Any  # HACK
+
     def __init__(self, gwy) -> None:
-        self._msgs = {}  # code, should be code/ctx? ?deprecate
-        self._msgz = {}  # code/verb/ctx, should be code/ctx/verb?
+        self._msgs: dict[_CodeT, Message] = {}  # code, should be code/ctx? ?deprecate
+        self._msgz: dict[_CodeT, Any] = {}  # code/verb/ctx, should be code/ctx/verb?
 
     def _handle_msg(self, msg: Message) -> None:  # TODO: beware, this is a mess
         """Store a msg in _msgs[code] (only latest I/RP) and _msgz[code][verb][ctx]."""
@@ -185,40 +113,44 @@ class MessageDB:
         """
         return [m for c in self._msgz.values() for v in c.values() for m in v.values()]
 
-    def _get_msg_by_hdr(self, hdr) -> Optional[Message]:
+    def _get_msg_by_hdr(self, hdr: _HeaderT) -> None | Message:
         """Return a msg, if any, that matches a header."""
 
-        code, verb, _, *args = hdr.split("|")
+        code, verb, _, *args = hdr.split("|")  # _ is device_id
 
         try:
-            if args and (ctx := args[0]):
+            if args and (ctx := args[0]):  # ctx may == True
                 msg = self._msgz[code][verb][ctx]
             elif False in self._msgz[code][verb]:
                 msg = self._msgz[code][verb][False]
             elif None in self._msgz[code][verb]:
                 msg = self._msgz[code][verb][None]
             else:
-                return
+                return None
         except KeyError:
-            return
+            return None
 
         if msg._pkt._hdr != hdr:
             raise LookupError
 
         return msg
 
-    def _msg_flag(self, code, key, idx) -> Optional[bool]:
+    def _msg_flag(self, code: _CodeT, key, idx) -> None | bool:
 
         if flags := self._msg_value(code, key=key):
             return bool(flags[idx])
+        return None
 
-    def _msg_value(self, code, *args, **kwargs):
+    def _msg_value(self, code: _CodeT, *args, **kwargs):
 
         if isinstance(code, (str, tuple)):  # a code or a tuple of codes
             return self._msg_value_code(code, *args, **kwargs)
+        # raise RuntimeError
         return self._msg_value_msg(code, *args, **kwargs)  # assume is a Message
 
-    def _msg_value_code(self, code, verb=None, key=None, **kwargs) -> Optional[dict]:
+    def _msg_value_code(
+        self, code: _CodeT, verb: _VerbT = None, key=None, **kwargs
+    ) -> None | dict | list:
 
         assert (
             not isinstance(code, tuple) or verb is None
@@ -239,23 +171,24 @@ class MessageDB:
 
         return self._msg_value_msg(msg, key=key, **kwargs)
 
-    @staticmethod  # FIXME: messy (uses msg, others use code - move to Message?)
-    def _msg_value_msg(msg, key=None, zone_idx=None, domain_id=None) -> Optional[dict]:
+    def _msg_value_msg(
+        self, msg: None | Message, key=None, zone_idx: str = None, domain_id: str = None
+    ) -> None | dict | list:
 
         if msg is None:
-            return
+            return None
         elif msg._expired:
-            _delete_msg(msg)
+            self._gwy._loop.call_soon(_delete_msg, msg)  # HA bugs without deferred call
 
-        if msg.code == _1FC9:  # NOTE: list of lists/tuples
+        if msg.code == Code._1FC9:  # NOTE: list of lists/tuples
             return [x[1] for x in msg.payload]
 
+        idx: None | str = None
+        val: None | str = None
         if domain_id:
             idx, val = SZ_DOMAIN_ID, domain_id
         elif zone_idx:
             idx, val = SZ_ZONE_IDX, zone_idx
-        else:
-            idx = val = None
 
         if isinstance(msg.payload, dict):
             msg_dict = msg.payload
@@ -266,7 +199,7 @@ class MessageDB:
             }
         else:
             # TODO: this isn't ideal: e.g. a controller is being treated like a 'stat
-            #  I 101 --:------ --:------ 12:126457 2309 006 0107D0-0207D0  # is a CTL
+            # .I 101 --:------ --:------ 12:126457 2309 006 0107D0-0207D0  # is a CTL
             msg_dict = msg.payload[0]
 
         assert (
@@ -282,45 +215,6 @@ class MessageDB:
         }
 
 
-class MessageDatabaseSql:
-    """Maintain/utilize an entity's state database."""
-
-    CREATE_TABLE = """
-        CREATE TABLE msgs (
-            hdr data_type PRIMARY KEY,
-            ctx data_type DEFAULT "",
-            dtm data_type NOT NULL UNIQUE,
-            msg data_type NOT NULL,
-        ) [WITHOUT ROWID];
-    """
-
-    def __init__(self, gwy) -> None:
-        self._cur = gwy._cur  # gwy._db.cursor()
-
-    def _handle_msg(self, msg: Message) -> None:  # TODO: beware, this is a mess
-        pass
-
-    @property
-    def _msg_db(self) -> list:  # a flattened version of _msgz[code][verb][indx]
-        pass
-
-    def _get_msg_by_hdr(self, hdr) -> Optional[Message]:
-        pass
-
-    def _msg_flag(self, code, key, idx) -> Optional[bool]:
-        pass
-
-    def _msg_value(self, code, *args, **kwargs):
-        pass
-
-    def _msg_value_code(self, code, verb=None, key=None, **kwargs) -> Optional[dict]:
-        pass
-
-    @staticmethod  # FIXME: messy (uses msg, others use code - move to Message?)
-    def _msg_value_msg(msg, key=None, zone_idx=None, domain_id=None) -> Optional[dict]:
-        pass
-
-
 class Discovery(MessageDB):
     MAX_CYCLE_SECS = 30
     MIN_CYCLE_SECS = 3
@@ -328,19 +222,18 @@ class Discovery(MessageDB):
     def __init__(self, gwy, *args, **kwargs) -> None:
         super().__init__(gwy, *args, **kwargs)
 
-        self._disc_tasks = None
+        self._disc_tasks: dict[_HeaderT, dict] = None  # type: ignore[assignment]
         self._disc_tasks_poller = None
 
         if not gwy.config.disable_discovery and isinstance(
             gwy.pkt_protocol, PacketProtocolPort
         ):  # TODO: here, or in get_xxx()?
-            # delay = 0  # random.uniform(0.5, 1.5)
             # gwy._loop.call_soon_threadsafe(
-            #     gwy._loop.call_later, delay, self._start_discovery_poller
+            #     gwy._loop.call_later, random(0.5, 1.5), self._start_discovery_poller
             # )
             gwy._loop.call_soon(self._start_discovery_poller)
 
-    def _setup_discovery_tasks(self) -> dict:
+    def _setup_discovery_tasks(self) -> None:
         raise NotImplementedError
 
     def _add_discovery_task(
@@ -395,18 +288,25 @@ class Discovery(MessageDB):
         command for later.
         """
 
-        def find_newer_msg(hdr: str, task: dict) -> Optional[Message]:
-            msgs = [self._get_msg_by_hdr(hdr[:5] + v + hdr[7:]) for v in (I_, RP)]
+        def find_newer_msg(hdr: _HeaderT, task: dict) -> None | Message:
+            msgs: list[Message] = [
+                m
+                for m in [self._get_msg_by_hdr(hdr[:5] + v + hdr[7:]) for v in (I_, RP)]
+                if m is not None
+            ]
 
             if (
-                (msgs := [m for m in msgs if m is not None])
+                msgs
                 and (msg := max(msgs))
                 and msg.dtm > task["next_due"] - task["interval"]
             ):
                 return msg
 
             # has the controller sent an array recently - use that instead?
-            for code in (_000A, _30C9):  # can't use 2309 (no mode) instead of 2349
+            for code in (
+                Code._000A,
+                Code._30C9,
+            ):  # can't use 2309 (no mode) instead of 2349
                 try:
                     if task["command"].code == code and (
                         msg := self.tcs._msgz[code][I_][True]
@@ -415,7 +315,9 @@ class Discovery(MessageDB):
                 except KeyError:
                     pass
 
-        def interval(hdr: str, failures: int) -> td:
+            return None
+
+        def interval(hdr: _HeaderT, failures: int) -> td:
             """Adjust the ineterval - backoff if any failures."""
 
             if failures > 5:
@@ -429,7 +331,7 @@ class Discovery(MessageDB):
 
             return td(seconds=secs)
 
-        async def send_disc_task(hdr: str, task: dict) -> Optional[Message]:
+        async def send_disc_task(hdr: _HeaderT, task: dict) -> None | Message:
             """Send a scheduled command and wait for/return the reponse."""
 
             try:
@@ -450,6 +352,8 @@ class Discovery(MessageDB):
             else:
                 return result
 
+            return None
+
         while True:
             if self._gwy.config.disable_discovery:
                 await asyncio.sleep(self.MIN_CYCLE_SECS)
@@ -491,13 +395,13 @@ class Entity(Discovery):
      - if the entity can Rx packets (e.g. can the HGI send it an RQ)
     """
 
-    _SLUG = None
+    _SLUG: str = None  # type: ignore[assignment]
 
     def __init__(self, gwy) -> None:
         super().__init__(gwy)
 
         self._gwy = gwy
-        self.id = None
+        self.id: _DeviceIdT = None  # type: ignore[assignment]
 
         self._qos_tx_count = 0  # the number of pkts Tx'd with no matching Rx
 
@@ -530,18 +434,18 @@ class Entity(Discovery):
     def _make_cmd(self, code, dest_id, payload="00", verb=RQ, **kwargs) -> None:
         self._send_cmd(self._gwy.create_cmd(verb, dest_id, code, payload, **kwargs))
 
-    def _send_cmd(self, cmd, **kwargs) -> None:
+    def _send_cmd(self, cmd, **kwargs) -> None | Future:
         if self._gwy.config.disable_sending:
             _LOGGER.info(f"{cmd} < Sending is disabled")
-            return
+            return None
 
         if self._qos_tx_count > _QOS_TX_LIMIT:
             _LOGGER.info(f"{cmd} < Sending is deprecated for {self}")
-            return
+            return None
 
         cmd._source_entity = self
         # self._msgs.pop(cmd.code, None)  # NOTE: Cause of DHW bug
-        self._gwy.send_cmd(cmd)  # BUG, should be: await async_send_cmd()
+        return self._gwy.send_cmd(cmd)  # BUG, should be: await async_send_cmd()
 
     @property
     def traits(self) -> dict:
@@ -576,7 +480,7 @@ def _delete_msg(msg) -> None:
             pass
 
 
-class Parent:  # A System, Zone, DhwZone or a UfhController
+class Parent(Entity):  # A System, Zone, DhwZone or a UfhController
     """A Parent can be a System (TCS), a heating Zone, a DHW Zone, or a UfhController.
 
     For a System, children include the appliance controller, the children of all Zones
@@ -589,20 +493,29 @@ class Parent:  # A System, Zone, DhwZone or a UfhController
     There is a `set_parent` method, but no `set_child` method.
     """
 
+    actuator_by_id: dict[_DeviceIdT, Entity]
+    actuators: list[Entity]
+
+    circuit_by_id: dict[str, Any]
+
+    _dhw_sensor: Entity
+    _dhw_valve: Entity
+    _htg_valve: Entity
+
     def __init__(self, *args, child_id: str = None, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        self._child_id = child_id
+        self._child_id: str = child_id  # type: ignore[assignment]
 
         # self._sensor: Child = None
-        self.child_by_id: Dict[str:Child] = {}
-        self.childs: List[Child] = []
+        self.child_by_id: dict[str, Child] = {}
+        self.childs: list[Child] = []
 
     def _handle_msg(self, msg: Message) -> None:
         def eavesdrop_ufh_circuits():
-            if msg.code == _22C9:
-                #  I --- 02:044446 --:------ 02:044446 22C9 024 00-076C0A28-01 01-06720A28-01 02-06A40A28-01 03-06A40A2-801  # NOTE: fragments
-                #  I --- 02:044446 --:------ 02:044446 22C9 006 04-07D00A28-01                                               # [{'ufh_idx': '04',...
+            if msg.code == Code._22C9:
+                # .I --- 02:044446 --:------ 02:044446 22C9 024 00-076C0A28-01 01-06720A28-01 02-06A40A28-01 03-06A40A2-801  # NOTE: fragments
+                # .I --- 02:044446 --:------ 02:044446 22C9 006 04-07D00A28-01                                               # [{'ufh_idx': '04',...
                 circuit_idxs = [c[SZ_UFH_IDX] for c in msg.payload]
 
                 for cct_idx in circuit_idxs:
@@ -634,10 +547,10 @@ class Parent:  # A System, Zone, DhwZone or a UfhController
         """Set the domain id, after validating it."""
         self._child_id = value
 
-    def _set_child(
+    def _add_child(
         self, child: Any, *, child_id: str = None, is_sensor: bool = None
     ) -> None:
-        """Add/Set the parent's child device, after validating the association.
+        """Add a child device to this Parent, after validating the association.
 
         Also sets various other parent-specific object references (e.g. parent._sensor).
 
@@ -645,7 +558,7 @@ class Parent:  # A System, Zone, DhwZone or a UfhController
         """
 
         # NOTE: here to prevent circular references
-        from .devices import (
+        from .device import (
             BdrSwitch,
             DhwSensor,
             OtbGateway,
@@ -654,8 +567,7 @@ class Parent:  # A System, Zone, DhwZone or a UfhController
             UfhCircuit,
             UfhController,
         )
-        from .systems import System
-        from .zones import DhwZone, Zone
+        from .system import DhwZone, System, Zone
 
         if hasattr(self, "childs") and child not in self.childs:  # Any parent
             assert isinstance(
@@ -687,8 +599,7 @@ class Parent:  # A System, Zone, DhwZone or a UfhController
 
         elif hasattr(self, SZ_CIRCUITS):  # UFH circuit
             assert isinstance(self, UfhController)  # TODO: remove me
-            if child not in self.circuits:
-                self.circuits.append(child)
+            if child not in self.circuit_by_id:
                 self.circuit_by_id[child.id] = child
 
         elif hasattr(self, SZ_ACTUATORS):  # HTG zone
@@ -751,7 +662,7 @@ class Parent:  # A System, Zone, DhwZone or a UfhController
             )
 
 
-class Child:  # A Zone, Device or a UfhCircuit
+class Child(Entity):  # A Zone, Device or a UfhCircuit
     """A Device can be the Child of a Parent (a System, a heating Zone, or a DHW Zone).
 
     A Device may/may not have a Parent, but all devices will have the gateway as a
@@ -768,13 +679,13 @@ class Child:  # A Zone, Device or a UfhCircuit
     ) -> None:
         super().__init__(*args, **kwargs)
 
-        self._parent: Parent = parent
+        self._parent = parent  # type: ignore[assignment]
+        self._is_sensor = is_sensor  # type: ignore[assignment]
 
-        self._child_id: str = None
-        self._is_sensor: bool = is_sensor
+        self._child_id: None | str = None  # TODO: should be: str?
 
     def _handle_msg(self, msg: Message) -> None:
-        from .devices import Controller, UfhController
+        from .device import Controller, UfhController
 
         def eavesdrop_parent_zone():
             if isinstance(msg.src, UfhController):
@@ -783,7 +694,7 @@ class Child:  # A Zone, Device or a UfhCircuit
             if SZ_ZONE_IDX not in msg.payload:
                 return
 
-            if msg.code in (_1060, _12B0, _2309, _3150):  # not 30C9
+            if msg.code in (Code._1060, Code._12B0, Code._2309, Code._3150):  # not 30C9
                 self.set_parent(msg.dst, child_id=msg.payload[SZ_ZONE_IDX])
 
         super()._handle_msg(msg)
@@ -799,11 +710,11 @@ class Child:  # A Zone, Device or a UfhCircuit
 
     def _get_parent(
         self, parent: Parent, *, child_id: str = None, is_sensor: bool = None
-    ) -> Tuple[Parent, str]:
+    ) -> tuple[Parent, None | str]:
         """Get the device's parent, after validating it."""
 
         # NOTE: here to prevent circular references
-        from .devices import (
+        from .device import (
             BdrSwitch,
             Controller,
             DhwSensor,
@@ -814,22 +725,21 @@ class Child:  # A Zone, Device or a UfhCircuit
             UfhCircuit,
             UfhController,
         )
-        from .systems import System
-        from .zones import DhwZone, Zone
+        from .system import DhwZone, System, Zone
 
         if isinstance(self, UfhController):
             child_id = FF
 
         if isinstance(parent, Controller):  # A controller cant be a Parent
-            parent: System = parent.tcs
+            parent: System = parent.tcs  # type: ignore[assignment, no-redef]
 
         if isinstance(parent, System) and child_id:
             if child_id in (F9, FA):
-                parent: DhwZone = parent.get_dhw_zone()
+                parent: DhwZone = parent.get_dhw_zone()  # type: ignore[no-redef]
             # elif child_id == FC:
             #     pass
-            elif int(child_id, 16) < self._gwy.config.max_zones:
-                parent: Zone = parent.get_htg_zone(child_id)
+            elif int(child_id, 16) < parent._max_zones:
+                parent: Zone = parent.get_htg_zone(child_id)  # type: ignore[no-redef, attr-defined]
 
         elif isinstance(parent, Zone) and not child_id:
             child_id = child_id or parent.idx
@@ -864,7 +774,7 @@ class Child:  # A Zone, Device or a UfhCircuit
         #     if self._child_id and self._child_id != child_id:
         #         raise CorruptStateError(f"child_id mismatch: {self._child_id} != {child_id}")
 
-        PARENT_RULES = {
+        PARENT_RULES: dict[Any, dict] = {
             DhwZone: {SZ_ACTUATORS: (BdrSwitch,), SZ_SENSOR: (DhwSensor,)},
             System: {
                 SZ_ACTUATORS: (BdrSwitch, OtbGateway, UfhController),
@@ -940,12 +850,11 @@ class Child:  # A Zone, Device or a UfhCircuit
         controller, or an UFH controller.
         """
 
-        from .devices import UfhController  # NOTE: here to prevent circular references
+        from .device import UfhController  # NOTE: here to prevent circular references
 
         parent, child_id = self._get_parent(
             parent, child_id=child_id, is_sensor=is_sensor
         )
-
         ctl = parent if isinstance(parent, UfhController) else parent.ctl
 
         if self.ctl and self.ctl is not ctl:
@@ -955,7 +864,7 @@ class Child:  # A Zone, Device or a UfhCircuit
                 "(or perhaps the device has multiple controllers?"
             )
 
-        parent._set_child(self, child_id=child_id, is_sensor=is_sensor)
+        parent._add_child(self, child_id=child_id, is_sensor=is_sensor)
         # parent.childs.append(self)
         # parent.child_by_id[self.id] = self
 
