@@ -5,7 +5,7 @@
 
 Helper functions.
 """
-from __future__ import annotations
+# from __future__ import annotations  # incompatible with @typechecked
 
 import ctypes
 import sys
@@ -66,7 +66,7 @@ def dt_str() -> str:
 
 
 @typechecked
-def bool_from_hex(value: str) -> Optional[bool]:  # either 00 or C8
+def bool_from_hex(value: str) -> Optional[bool]:  # either False, True or None
     """Convert a 2-char hex string into a boolean."""
     if not isinstance(value, str) or len(value) != 2:
         raise ValueError(f"Invalid value: {value}, is not a 2-char hex string")
@@ -75,6 +75,16 @@ def bool_from_hex(value: str) -> Optional[bool]:  # either 00 or C8
     return {"00": False, "C8": True}[value]
 
 
+@typechecked
+def bool_to_hex(value: Optional[bool]) -> str:  # either 00, C8 or FF
+    """Convert a boolean into a 2-char hex string."""
+    if value is None:
+        return "FF"
+    if not isinstance(value, bool):
+        raise ValueError(f"Invalid value: {value}, is not bool")
+    return {False: "00", True: "C8"}[value]
+
+
 @typechecked
 def date_from_hex(value: str) -> Optional[str]:  # YY-MM-DD
     """Convert am 8-char hex string into a date, format YY-MM-DD."""
@@ -90,7 +100,7 @@ def date_from_hex(value: str) -> Optional[str]:  # YY-MM-DD
 
 
 @typechecked
-def double(value: str, factor: int = 1) -> Optional[float]:
+def double_from_hex(value: str, factor: int = 1) -> Optional[float]:
     """Convert a 4-char hex string into a double."""
     if not isinstance(value, str) or len(value) != 4:
         raise ValueError(f"Invalid value: {value}, is not a 4-char hex string")
@@ -99,6 +109,16 @@ def double(value: str, factor: int = 1) -> Optional[float]:
     return int(value, 16) / factor
 
 
+@typechecked
+def double_to_hex(value: Optional[float], factor: int = 1) -> str:
+    """Convert a double into 4-char hex string."""
+    if value is None:
+        return "7FFF"
+    if not isinstance(value, float):
+        raise ValueError(f"Invalid value: {value}, is not a double (a float)")
+    return f"{int(value * factor):04X}"
+
+
 @typechecked
 def dtm_from_hex(value: str) -> Optional[str]:  # from parsers
     """Convert a 12/14-char hex string to an isoformat datetime (naive, local)."""
@@ -107,7 +127,7 @@ def dtm_from_hex(value: str) -> Optional[str]:  # from parsers
 
     if not isinstance(value, str) or len(value) not in (12, 14):
         raise ValueError(f"Invalid value: {value}, is not a 12/14-char hex string")
-    if value == "FF" * 6:
+    if value[-12:] == "FF" * 6:
         return None
     if len(value) == 12:
         value = f"00{value}"
@@ -204,7 +224,9 @@ def flag8(byte: str, lsb: bool = False) -> list:  # TODO: should be tuple
 
 # TODO: add a wrapper for EF, & 0xF0
 @typechecked
-def percent(value: str, high_res: bool = True) -> Optional[float]:  # c.f. valve_demand
+def percent_from_hex(
+    value: str, high_res: bool = True
+) -> Optional[float]:  # c.f. valve_demand
     """Convert a 2-char hex string into a percentage.
 
     The range is 0-100%, with resolution of 0.5% (high_res) or 1%.
@@ -264,13 +286,13 @@ def temp_to_hex(value: Optional[float]) -> str:
     if not isinstance(value, (float, int)):
         raise TypeError(f"Invalid temp: {value} is not a float")
     if not -(2**7) <= value < 2**7:  # TODO: tighten range
-        raise ValueError(f"Invalid temp: {value:02X} is out of range")
+        raise ValueError(f"Invalid temp: {value} is out of range")
     temp = int(value * 100)
     return f"{temp if temp >= 0 else temp + 2 ** 16:04X}"
 
 
 @typechecked
-def valve_demand(value: str) -> Optional[dict]:  # c.f. percent()
+def valve_demand(value: str) -> Optional[dict]:  # c.f. percent_from_hex()
     """Convert a 2-char hex string into a percentage.
 
     The range is 0-100%, with resolution of 0.5% (high_res) or 1%.
